function [U,Vx,Wy,Xm,Ym]=ccaApca(X,Y,K);
%CCAAPCA  Calculate CCA of augmented PCA coefficient vectors.
%  [U,Vx,Wy,Xm,Ym] = ccaApca(X,Y,K) calculates K+c dimensional augmented
%  principal subspace of input images given in columns of X, where c is dim(Y).
%  This subspace is determined by K+c basis vectors U and by the origin (mean vector) Xm.
%  It also performs CCA on vectors of principal components and it returns c
%  canonical correlation vectors Vx of dimension K+c (the origin of this subspace is in 0),
%  and c canonical vectors Wy of dimension c with the origin of the subspace in Ym.

[M,N]=size(X);
c=min(M,size(Y,1));

%PCA
[Xm,U]=pca(X,N-1);
A=is2fs(X,Xm,U);

%CCA on N-1 coef.
[Vx,Wy,Xm0,Ym]=cca(A,Y);

%append PCA basis
gp=sqrt(sum(Vx(K+1:N-1,:).^2));
Fp=repmat(1./gp,M,1).*(U(:,K+1:N-1)*Vx(K+1:N-1,:));
F=orth(Fp);
G=repmat(gp,c,1).*(F'*Fp);
U=[U(:,1:K) F];
Vx=[Vx(1:K,:);G];




