%demoCCA

disp('*******************************************************************');
disp('* Demo of robust estimation of canonical correlation coefficients *')
disp('*******************************************************************');

%load training data
load data1 %load images X,Xt0,Xt
idxs=[0:12:359];  %training orientations
idxst=setdiff([0:3:359],idxs); %test orientations
Y=[sin(idxs*pi/180);cos(idxs*pi/180)]; %trigonometric representation
Yt=[sin(idxst*pi/180);cos(idxst*pi/180)];
[M,N]=size(X);

figure;dispImgs(X);set(gcf,'Name','X - training images');
disp('Displaying training images. Press any key to continue.');
pause

figure;dispImgs(Xt0);set(gcf,'Name','Xt0 - non-occluded test images');
disp('Displaying non-occluded test images. Press any key to continue.');
pause

figure;dispImgs(Xt);set(gcf,'Name','Xt - occluded test images');
disp('Displaying occluded test images. Press any key to continue.');
pause

K=10;

% P C A
%calculate principal components (PCA)
[Xm,U,L]=pca(X,N-1);
A=is2fs(X,Xm,U);
At0=is2fs(Xt0,Xm,U);
At=is2fs(Xt,Xm,U);


%***************** Non-occluded test images *********************

%***** CCA on GT *****
%calculate canonical correlation vectors (CCA)
[Wx1,Wy1,Xm1,Ym1]=cca(X,Y);
%calculate coeffitients
Ax1=is2fs(X,Xm1,Wx1);
Ay1=is2fs(Y,Ym1,Wy1);
Axt1=is2fs(Xt0,Xm1,Wx1);
Ayt1=is2fs(Yt,Ym1,Wy1);
%map from FSx to ISy
f1=Y/Ax1;
Y1=f1*Ax1;
Yt1=f1*Axt1;
%map from ISy to angles
ang=atan2(Y1(1,:),Y1(2,:))*180/pi+180;angl=size(ang,2);
R1=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt1(1,:),Yt1(2,:))*180/pi+180;angl=size(ang,2);
Rt1=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R1,'r.');hold on; scatter(idxst,Rt1,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','CCA on GT');
xlabel('actual orientation');ylabel('detected orientation');title('CCA on GT');
disp('Displaying results of CCA on non-occluded test images.');
circD1=mean(min(abs([Rt1-idxst;0+Rt1-idxst;360+Rt1-idxst;0+idxst-Rt1;360+idxst-Rt1])));
disp(['mean circular distance: ', num2str(circD1)]);
disp('Press any key to continue.');
pause


%***** CCA on full PC of GT *****
%calculate canonical vectors of principal components
[Vx2,Wy2,Xm2,Ym2]=cca(A,Y);
%calculate coeffitients
Ax2=is2fs(A,Xm2,Vx2);
Ay2=is2fs(Y,Ym2,Wy2);
Axt2=is2fs(At0,Xm2,Vx2);
Ayt2=is2fs(Yt,Ym2,Wy2);
%map from FSx to ISy
f2=Y/Ax2;
Y2=f2*Ax2;
Yt2=f2*Axt2;
%map from ISy to angles
ang=atan2(Y2(1,:),Y2(2,:))*180/pi+180;angl=size(ang,2);
R2=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt2(1,:),Yt2(2,:))*180/pi+180;angl=size(ang,2);
Rt2=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R2,'r.');hold on; scatter(idxst,Rt2,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','CCA on full PC of GT');
xlabel('actual orientation');ylabel('detected orientation');title('CCA on full PC of GT');
disp('Displaying results of CCA on full PC of non-occluded test images.');
circD2=mean(min(abs([Rt2-idxst;0+Rt2-idxst;360+Rt2-idxst;0+idxst-Rt2;360+idxst-Rt2])));
disp(['mean circular distance: ', num2str(circD2)]);
disp('Press any key to continue.');
pause


%***** CCA on trunc. PC of GT *****
%calculate canonical vectors of principal components
[Vx3,Wy3,Xm3,Ym3]=cca(A(1:K,:),Y);
%calculate coeffitients
Ax3=is2fs(A(1:K,:),Xm3,Vx3);
Ay3=is2fs(Y,Ym3,Wy3);
Axt3=is2fs(At0(1:K,:),Xm3,Vx3);
Ayt3=is2fs(Yt,Ym3,Wy3);
%map from FSx to ISy
f3=Y/Ax3;
Y3=f3*Ax3;
Yt3=f3*Axt3;
%map from ISy to angles
ang=atan2(Y3(1,:),Y3(2,:))*180/pi+180;angl=size(ang,2);
R3=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt3(1,:),Yt3(2,:))*180/pi+180;angl=size(ang,2);
Rt3=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R3,'r.');hold on; scatter(idxst,Rt3,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','CCA on trunc. PC of GT');
xlabel('actual orientation');ylabel('detected orientation');title('CCA on trunc. PC of GT');
disp('Displaying results of CCA on trunc. PC of non-occluded test images.');
circD3=mean(min(abs([Rt3-idxst;0+Rt3-idxst;360+Rt3-idxst;0+idxst-Rt3;360+idxst-Rt3])));
disp(['mean circular distance: ', num2str(circD3)]);
disp('Press any key to continue.');
pause


%***** CCA on augm. trunc. PC of GT *****
[U4,Vx4,Wy4,Xm4,Ym4]=ccaApca(X,Y,K);
%calculate coeffitients
A4=is2fs(X,Xm4,U4);
Ax4=is2fs(A4,0,Vx4);
At4=is2fs(Xt0,Xm,U4);
Axt4=is2fs(At4,0,Vx4);
Ay4=is2fs(Y,Ym4,Wy4);
Ayt4=is2fs(Yt,Ym4,Wy4);
%map from FSx to ISy
f4=Y/Ax4;
Y4=f4*Ax4;
Yt4=f4*Axt4;
%map from ISy to angles
ang=atan2(Y4(1,:),Y4(2,:))*180/pi+180;angl=size(ang,2);
R4=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt4(1,:),Yt4(2,:))*180/pi+180;angl=size(ang,2);
Rt4=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R4,'r.');hold on; scatter(idxst,Rt4,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','CCA on augm. trunc. PC of GT');
xlabel('actual orientation');ylabel('detected orientation');title('CCA on augm. trunc. PC of GT');
disp('Displaying results of CCA on augm. trunc. PC of non-occluded test images.');
circD4=mean(min(abs([Rt4-idxst;0+Rt4-idxst;360+Rt4-idxst;0+idxst-Rt4;360+idxst-Rt4])));
disp(['mean circular distance: ', num2str(circD4)]);
disp('Press any key to continue.');
pause


%***************** Occluded test images *********************

%***** nonrobust CCA *****
%calculate canonical vectors (CCA)
[Wx5,Wy5,Xm5,Ym5]=cca(X,Y);
%calculate coeffitients
Ax5=is2fs(X,Xm5,Wx5);
Ay5=is2fs(Y,Ym5,Wy5);
Axt5=is2fs(Xt,Xm5,Wx5);
Ayt5=is2fs(Yt,Ym5,Wy5);
%map from FSx to ISy
f5=Y/Ax5;
Y5=f5*Ax5;
Yt5=f5*Axt5;
%map from ISy to angles
ang=atan2(Y5(1,:),Y5(2,:))*180/pi+180;angl=size(ang,2);
R5=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt5(1,:),Yt5(2,:))*180/pi+180;angl=size(ang,2);
Rt5=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R5,'r.');hold on; scatter(idxst,Rt5,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','CCA on occl.');
xlabel('actual orientation');ylabel('detected orientation');title('CCA on occl. test images');
disp('Displaying results of CCA on occluded test images.');
circD5=mean(min(abs([Rt5-idxst;0+Rt5-idxst;360+Rt5-idxst;0+idxst-Rt5;360+idxst-Rt5])));
disp(['mean circular distance: ', num2str(circD5)]);
disp('Press any key to continue.');
pause


%***** robust CCA on augmented trunc. PC *****
disp('Estimating robust canonical correlation coefficients. This may take a while...');
[U6,Vx6,Wy6,Xm6,Ym6]=ccaApca(X,Y,K);
%calculate coeffitients
A6=is2fs(X,Xm6,U6);
Ax6=is2fs(A6,0,Vx6);
Ay6=is2fs(Y,Ym6,Wy6);
At6=is2fsRob(Xt,Xm,U6,L,300,0.95,50,2);
Axt6=is2fs(At6,0,Vx6);
Ayt6=is2fs(Yt,Ym6,Wy6);
%map from FSx to ISy
f6=Y/Ax6;
Y6=f6*Ax6;
Yt6=f6*Axt6;
%map from ISy to angles
ang=atan2(Y6(1,:),Y6(2,:))*180/pi+180;angl=size(ang,2);
R6=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];
ang=atan2(Yt6(1,:),Yt6(2,:))*180/pi+180;angl=size(ang,2);
Rt6=[ang(1,(angl/2+1):angl) ang(1,1:angl/2)];

%display results
figure;scatter(idxs,R6,'r.');hold on; scatter(idxst,Rt6,'b.');hold off;
axis([0 360 -50 400]);set(gcf,'Name','rob. CCA on occl.');
xlabel('actual orientation');ylabel('detected orientation');title('robust CCA on occl. test images');
disp('Displaying results of robust CCA on occluded test images.')
circD6=mean(min(abs([Rt6-idxst;0+Rt6-idxst;360+Rt6-idxst;0+idxst-Rt6;360+idxst-Rt6])));
disp(['mean circular distance: ', num2str(circD6)]);




return

if 1==0
   %load images
   STEPD=12;
   STEPDt=3;
   idxs=[0:STEPD:359];
   idxst=setdiff([0:STEPDt:359],idxs);
   
   X=loadImgs('c:/danijels/Matlab/data/rsbaza/o02_','%03d','G.bmp',idxs);
   Y=[sin(idxs*pi/180);cos(idxs*pi/180)];
   Xt0=loadImgs('c:/danijels/Matlab/data/rsbaza/o02_','%03d','G.bmp',idxst);
   Yt=[sin(idxst*pi/180);cos(idxst*pi/180)];
   
   X=resizeImgs(X,.5);
   Xt0=resizeImgs(Xt0,.5);
   Xt=addOL(Xt0,2,.15,.999999);
   
   [M,N]=size(X);
end
