function [W,Ms]=lda(A,c,n)
%LDA  Linear Discriminant Analysis.
%  [W,Ms]=lda(A,c,n) calculates LDA eigenvectors W from the training images arranged
%  in columns of X in c groups of n images, thus c is the number of classes and
%  n is the number of images in each class. It returns also the matrix Ms, which
%  contains c (c-1)-dimensional means of all c classes in LDA subspace.

SB=0; SW=0;
opts.disp = 0;
MM=mean(A')';
for i=1:c
   Ms(:,i)=mean(A(:,(i-1)*n+1:i*n)')';  %mean values of the classes in input space
   SB=SB+n*(Ms(:,i)-MM)*(Ms(:,i)-MM)';  % between classes scatter matrix
   for j=1:n
      SW=SW+(A(:,(i-1)*n+j)-Ms(:,i))*(A(:,(i-1)*n+j)-Ms(:,i))';  % within class scatter matrix
   end;
end;

[W, EIGD] = eigs(inv(SW)*SB, c-1,'LM',opts);  % the solution of a generalized eigenproblem: 
                                              %         S_B w=lambda S_W w
Ms=W'*Ms;  %map means into the LDA space