function [U,Xm,W,Ms]=ldaApca(X,c,n,K);
%LDAAPCA  Calculate LDA of augmented PCA coefficient vectors.
%  [U,Xm,W,Ms] = ldaApca(X,c,n,K) calculates K+c-1 dimensional augmented
%  principal subspace of input images given in columns of X, where c is the number 
%  of classes and n the number of training images in each class.
%  This subspace is determined by K+c-1 basis vectors U and by the origin (mean vector) Xm.
%  It also performs LDA on vectors of principal components and it returns c-1
%  LDA eigenvectors W of dimension K+c-1 and c (c-1)-dimensional means of all c classes
%  in LDA subspace.

[M,N]=size(X);

%PCA
[Xm,U]=pca(X,N-1);
A=is2fs(X,Xm,U);

%LDA on N-1 coef.
[W,Ms]=lda(A,c,n); 

%append PCA basis
gp=sqrt(sum(W(K+1:N-1,:).^2));
Fp=repmat(1./gp,M,1).*(U(:,K+1:N-1)*W(K+1:N-1,:));
F=orth(Fp);
G=repmat(gp,c-1,1).*(F'*Fp);
U=[U(:,1:K) F];
W=[W(1:K,:);G];




