function [Xm,U,L]=pca(X,K);
%PCA  Principal Component Analysis.
%   [Xm,U,L] = pca(X) creates an eigenspace from data matrix X.
%   It returns mean image Xm, eigenvectors in columns of U and eigenvalues L.
%   Each column of X is a vector representing an image.
%   [Xm,U,L] = pca(X,K) returns only the first K eigenvectors and eigenvalues.

[M N]=size(X);
Xm=mean(X,2);
Xd=X-repmat(Xm,1,N);
if (N < M) %less images than image length
  C=Xd'*Xd;
  [V D Vt]=svd(C);
  U=Xd*V;
  U=U./repmat(sqrt(diag(D)'),M,1);
else %more images than image length
  C=Xd*Xd';
  [U D Ut]=svd(C);
end;
L=diag(D)'/N;

if nargin>1
   U=U(:,1:K);
   L=L(1:K);
end;   